/*-----------------------------------------------------------------------+
 | edu.tum.cs.ccts.model.base
 |                                                                       |
   $Id$            
 |                                                                       |
 | Copyright (c)  2004-2008 Technische Universitaet Muenchen             |
 |                                                                       |
 | Technische Universitaet Muenchen               #########  ##########  |
 | Institut fuer Informatik - Lehrstuhl IV           ##  ##  ##  ##  ##  |
 | Prof. Dr. Manfred Broy                            ##  ##  ##  ##  ##  |
 | Boltzmannstr. 3                                   ##  ##  ##  ##  ##  |
 | 85748 Garching bei Muenchen                       ##  ##  ##  ##  ##  |
 | Germany                                           ##  ######  ##  ##  |
 +-----------------------------------------------------------------------*/
package org.fortiss.tooling.kernel.internal.editor;

import org.conqat.ide.commons.ui.logging.LoggingUtils;
import org.eclipse.core.resources.IMarker;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.impl.AdapterImpl;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.part.MultiPageEditorPart;
import org.eclipse.ui.views.properties.IPropertySheetPage;
import org.eclipse.ui.views.properties.tabbed.ITabbedPropertySheetPageContributor;
import org.eclipse.ui.views.properties.tabbed.TabbedPropertySheetPage;
import org.fortiss.tooling.kernel.ToolingKernelActivator;
import org.fortiss.tooling.kernel.base.EditorBindingBase;
import org.fortiss.tooling.kernel.interfaces.IHandler;
import org.fortiss.tooling.kernel.services.IEditorService;

/**
 * This editor is used for displaying multiple editors provided by editor
 * bindings for a given model element.
 * 
 * @author hoelzlf
 * @author $Author$
 * @version $Rev$
 * @levd.rating YELLOW Hash: 5BCA3453180427D26544F7CF84AFDB4B
 */
public class BindingEditor extends MultiPageEditorPart implements
		ITabbedPropertySheetPageContributor {

	/** The editor's ID. */
	public static String ID = BindingEditor.class.getName();

	// TODO (FH): change this
	/** The tabbed property sheet contributor ID. */
	public static final String TABBED_PROPERTY_CONTRIBUTOR_ID = "edu.tum.cs.ccts.model.base.properties.Contributor";

	/**
	 * The object shown in this editor. This is valid as soon as
	 * {@link #init(IEditorSite, IEditorInput)} has been called.
	 */
	protected EObject editedObject;

	/** The model element handler to be used with the {@link #editedObject}. */
	protected IHandler<EObject> handler;

	/**
	 * The adapter used for learning about changes in the {@link #editedObject}.
	 */
	private final Adapter editedObjectChangeAdapter = new AdapterImpl() {

		/** {@inheritDoc} */
		@Override
		public void notifyChanged(Notification msg) {
			editedObjectChanged();
		}
	};

	/**
	 * This is called whenever something about the currently edited object
	 * changes. This is used to update the part name and to close the editor if
	 * the object is removed.
	 */
	protected void editedObjectChanged() {
		setPartName(handler.getName(editedObject));
		setContentDescription(handler.getDescription(editedObject));

		if (editedObject.eContainer() == null) {
			getSite().getPage().closeEditor(this, false);
		}
	}

	/** {@inheritDoc} */
	@Override
	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		if (!(input instanceof ModelElementEditorInput)) {
			throw new PartInitException(
					"BindingEditor initialized with wrong editor input object.");
		}
		super.init(site, input);

		ModelElementEditorInput meInput = ((ModelElementEditorInput) input);
		editedObject = meInput.getEditorInput();
		if (editedObject == null) {
			throw new PartInitException("Missing model element!");
		}

		handler = meInput.getModelElementHandler();
		if (handler == null) {
			throw new PartInitException("Missing model element handler!");
		}

		setPartName(handler.getName(editedObject));
		setContentDescription(handler.getDescription(editedObject));
		setTitleImage(handler.getIcon());

		editedObject.eAdapters().add(editedObjectChangeAdapter);
	}

	/** {@inheritDoc} */
	@Override
	protected void createPages() {
		int pageIndex = 0;
		for (EditorBindingBase<EObject> editorBinding : IEditorService.INSTANCE
				.getBindings(editedObject)) {
			try {
				IEditorPart editorPart = editorBinding.getEditorClass()
						.newInstance();
				addPage(editorPart, getEditorInput());
				setPageText(pageIndex++, editorBinding.getLabel());
			} catch (Exception ex) {
				LoggingUtils.error(ToolingKernelActivator.getDefault(),
						"Editor instantiation failed.", ex);
			}
		}
	}

	/** {@inheritDoc} */
	@Override
	public void doSave(IProgressMonitor monitor) {
		// Saving is handled automatically by emfStore
	}

	/** {@inheritDoc} */
	@Override
	public void doSaveAs() {
		// Saving is handled automatically by emfStore
	}

	/** {@inheritDoc} */
	@Override
	public void dispose() {
		editedObject.eAdapters().remove(editedObjectChangeAdapter);
		super.dispose();
	}

	/** {@inheritDoc} */
	@Override
	public boolean isDirty() {
		// editors for emfStore objects are never dirty
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isSaveAsAllowed() {
		return false;
	}

	/** {@inheritDoc} */
	@Override
	public boolean isSaveOnCloseNeeded() {
		return false;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String getContributorId() {
		return TABBED_PROPERTY_CONTRIBUTOR_ID;
	}

	/**
	 * {@inheritDoc}
	 */
	@SuppressWarnings("rawtypes")
	@Override
	public Object getAdapter(Class adapter) {
		if (adapter == IPropertySheetPage.class) {
			return new TabbedPropertySheetPage(this);
		}
		return super.getAdapter(adapter);
	}

	/**
	 * TODO: COMMENT ME
	 */
	public void gotoMarker(IMarker marker) {
		// TODO (FH): implement marker goto
	}
}
